<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns="http://www.w3.org/1999/xhtml" exclude-result-prefixes="#default">
    <!-- Escape URIs -->
    <!-- -->
    <xsl:variable name="ascii-charset"> !&quot;#$%&amp;&apos;()*+,-./0123456789:;&lt;=&gt;?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~&#127;&#128;&#129;&#130;&#131;&#132;&#133;&#134;&#135;&#136;&#137;&#138;&#139;&#140;&#141;&#142;&#143;&#144;&#145;&#146;&#147;&#148;&#149;&#150;&#151;&#152;&#153;&#154;&#155;&#156;&#157;&#158;&#159;&#160;&#161;&#162;&#163;&#164;&#165;&#166;&#167;&#168;&#169;&#170;&#171;&#172;&#173;&#174;&#175;&#176;&#177;&#178;&#179;&#180;&#181;&#182;&#183;&#184;&#185;&#186;&#187;&#188;&#189;&#190;&#191;&#192;&#193;&#194;&#195;&#196;&#197;&#198;&#199;&#200;&#201;&#202;&#203;&#204;&#205;&#206;&#207;&#208;&#209;&#210;&#211;&#212;&#213;&#214;&#215;&#216;&#217;&#218;&#219;&#220;&#221;&#222;&#223;&#224;&#225;&#226;&#227;&#228;&#229;&#230;&#231;&#232;&#233;&#234;&#235;&#236;&#237;&#238;&#239;&#240;&#241;&#242;&#243;&#244;&#245;&#246;&#247;&#248;&#249;&#250;&#251;&#252;&#253;&#254;&#255;</xsl:variable>
    <xsl:variable name="uri-ok">-_.!~*&apos;()0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="hex">0123456789ABCDEFabcdef</xsl:variable>
    <!-- -->
    <xsl:template name="tif-escape-uri">
        <xsl:param name="str"/>
        <xsl:param name="allow-utf8"/>
        <xsl:if test="$str">
            <xsl:variable name="first-char" select="substring($str,1,1)"/>
            <xsl:choose>
                <xsl:when test="$first-char = '%' and string-length($str) &gt;= 3 and contains($hex, substring($str,2,1)) and contains($hex, substring($str,3,1))">
                    <!-- The percent char is ok IF it followed by a valid hex pair -->
                    <xsl:value-of select="$first-char"/>
                </xsl:when>
                <xsl:when test="contains($uri-ok, $first-char)">
                    <!-- This char is ok inside urls -->
                    <xsl:value-of select="$first-char"/>
                </xsl:when>
                <xsl:when test="not(contains($ascii-charset, $first-char))">
                    <!-- Non-ascii output raw based on utf8 allowed or not -->
                    <xsl:choose>
                        <xsl:when test="$allow-utf8">
                            <xsl:value-of select="$first-char"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>%3F</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <!-- URL escape this char -->
                    <xsl:variable name="ascii-value" select="string-length(substring-before($ascii-charset,$first-char)) + 32"/>
                    <xsl:text>%</xsl:text>
                    <xsl:value-of select="substring($hex,floor($ascii-value div 16) + 1,1)"/>
                    <xsl:value-of select="substring($hex,$ascii-value mod 16 + 1,1)"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:call-template name="tif-escape-uri">
                <xsl:with-param name="str" select="substring($str,2)"/>
                <xsl:with-param name="allow-utf8" select="$allow-utf8"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    <!-- -->
</xsl:stylesheet>
